﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;
using ArcGIS.Desktop.Mapping.Voxel;

namespace VoxelLayerAPI.Examples.IsoSurface
{
	/// <summary>
	/// Illustrates how to create an isosurface
	/// </summary>
	internal class CreateIsoSurface : Button
	{
		protected override void OnClick()
		{
			//Selected voxel layer
			var voxelLayer = MapView.Active.GetSelectedLayers().OfType<VoxelLayer>().FirstOrDefault();
			if (voxelLayer == null)
			{
				voxelLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<VoxelLayer>().FirstOrDefault();
				if (voxelLayer == null)
					return;
			}

			QueuedTask.Run(() =>
			{
				var selProfile = voxelLayer.SelectedVariableProfile;

				//Visualization must be surface or CanCreateIsosurface will be false
				if (voxelLayer.Visualization != VoxelVisualization.Surface)
					voxelLayer.SetVisualization(VoxelVisualization.Surface);

				//Includes check for MaxNumberofIsoSurfaces
				if (selProfile.CanCreateIsosurface)
				{
					//use color range
					var min = ((CIMVoxelStretchRenderer)selProfile.Renderer).ColorRangeMin;
					var max = ((CIMVoxelStretchRenderer)selProfile.Renderer).ColorRangeMax;
					var mid = (max + min) / 2;

					//Create the iso surface
					var suffix = Math.Truncate(mid * 100) / 100;
					selProfile.CreateIsosurface(new IsosurfaceDefinition()
					{
						Name = $"Surface {suffix}",
						Value = mid,
						IsVisible = true
					});
				}
			});
		}
	}
}
